<?php

class ControllerPaymentMkb extends Controller
{
	private $error = array();

	public function index()
	{
		$data = [];
		$this->load->language('payment/mkb');

		$this->document->setTitle($this->language->get('heading_title'));

		$this->load->model('setting/setting');

		if (($this->request->server['REQUEST_METHOD'] == 'POST') && $this->validate()) {
			$this->model_setting_setting->editSetting('mkb', $this->request->post);

			$this->session->data['success'] = $this->language->get('text_success');

			$this->response->redirect($this->url->link('extension/payment', 'token=' . $this->session->data['token'], true));
		}

		$data['heading_title'] = $this->language->get('heading_title');

		$data['text_enabled']   = $this->language->get('text_enabled');
		$data['help_total']     = $this->language->get('help_total');
		$data['text_disabled']  = $this->language->get('text_disabled');
		$data['text_all_zones'] = $this->language->get('text_all_zones');

		$data['entry_order_status'] = $this->language->get('entry_order_status');
		$data['entry_geo_zone']     = $this->language->get('entry_geo_zone');

		$data['text_edit'] = $this->language->get('text_edit');

		$data['button_save']   = $this->language->get('button_save');
		$data['button_cancel'] = $this->language->get('button_cancel');

		$data['text_mode_test'] = $this->language->get('text_mode_test');
		$data['text_mode_prod'] = $this->language->get('text_mode_prod');

		if (isset($this->error['warning'])) {
			$data['error_warning'] = $this->error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset($this->error['paymentname'])) {
			$data['error_paymentname'] = $this->error['paymentname'];
		} else {
			$data['error_paymentname'] = '';
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => $this->language->get('text_home'),
			'href' => $this->url->link('common/home', 'token=' . $this->session->data['token'], true),
		);

		$data['breadcrumbs'][] = array(
			'text' => $this->language->get('text_extension'),
			'href' => $this->url->link('extension/payment', 'token=' . $this->session->data['token'], true),
		);

		$data['breadcrumbs'][] = array(
			'text' => $this->language->get('heading_title'),
			'href' => $this->url->link('payment/mkb', 'token=' . $this->session->data['token'], true),
		);

		$data['action'] = $this->url->link('payment/mkb', 'token=' . $this->session->data['token'], true);

		$data['cancel'] = $this->url->link('extension/payment', 'token=' . $this->session->data['token'], true);

		$settings = array(
			'paymentname'     => '',
			'mid'             => '',
			'secret'          => '',
			'login'           => '',
			'password'        => '',
			'mode'            => 'test',
			'merchant_mail'   => '',
			'total'           => '',
			'order_status_id' => '5', //Complete
			'geo_zone_id'     => 0,
			'status'          => 0,
			'sort_order'      => '',
		);

		foreach ($settings as $key => $default) {
			$data['entry_' . $key] = $this->language->get('entry_' . $key);

			if (isset($this->request->post['mkb_' . $key])) {
				$data['mkb_' . $key] = $this->request->post['mkb_' . $key];
			} elseif ($this->config->has('mkb_' . $key)) {
				$data['mkb_' . $key] = $this->config->get('mkb_' . $key);
			} else {
				$data['mkb_' . $key] = $default;
			}
		}

		$this->load->model('localisation/order_status');

		$data['order_statuses'] = $this->model_localisation_order_status->getOrderStatuses();

		$this->load->model('localisation/geo_zone');

		$data['geo_zones'] = $this->model_localisation_geo_zone->getGeoZones();

		$data['header']      = $this->load->controller('common/header');
		$data['column_left'] = $this->load->controller('common/column_left');
		$data['footer']      = $this->load->controller('common/footer');

		$this->response->setOutput($this->load->view('payment/mkb.tpl', $data));
	}

	protected function validate()
	{
		if (!$this->user->hasPermission('modify', 'payment/mkb')) {
			$this->error['warning'] = $this->language->get('error_permission');
		}

		if (!$this->request->post['mkb_paymentname']) {
			$this->error['paymentname'] = $this->language->get('error_paymentname');
		}

		return !$this->error;
	}

	public function install()
	{
		$this->db->query("
			CREATE TABLE IF NOT EXISTS `" . DB_PREFIX . "mkb` (
				`OrderID` int(11) NOT NULL,
				`amt` varchar(12) NOT NULL,
				`ResponseCode` int(11) NOT NULL,
				`AuthCode` varchar(8) DEFAULT NULL,
				`AcqID` varchar(6) NOT NULL,
				`fio` varchar(100) DEFAULT NULL,
				`ReasonCode` int(11) NOT NULL,
				`MerID` varchar(15) NOT NULL,
				`PaddedCardNo` varchar(18) DEFAULT NULL,
				`ReasonCodeDesc` varchar(100) NOT NULL,
				`ReferenceNo` varchar(12) DEFAULT NULL,
				`clientID` varchar(32) DEFAULT NULL,
				`cardID` varchar(32) DEFAULT NULL,
				`status` int(11) NOT NULL DEFAULT '0',
				PRIMARY KEY (OrderID)
		)");

	}

	public function uninstall()
	{
		$this->db->query(' drop table IF EXISTS `' . DB_PREFIX . 'mkb`');
	}
}
