<?php

use Mkb\Sdk\Client;
use Mkb\Sdk\Signature;
use Mkb\Sdk\Status;

class ControllerPaymentMkbTransactions extends Controller
{
	private $error = array();

	private $mkblib = null;

	public function __construct($registry)
	{

		$this->registry = $registry;
		if (method_exists($this->load, "library")) {
			$this->load->library('mkblib');
		}
		$this->mkblib = new Mkblib($this->registry);
	}

	public function index()
	{
		$data = [];
		$this->load->language('payment/mkb/transactions');
		$this->load->model('payment/mkb/transactions');

		$this->document->setTitle($this->language->get('heading_title'));

		if (isset($this->request->get['filter_order_id'])) {
			$filter_order_id = $this->request->get['filter_order_id'];
		} else {
			$filter_order_id = null;
		}

		if (isset($this->request->get['sort'])) {
			$sort = $this->request->get['sort'];
		} else {
			$sort = 'o.order_id';
		}

		if (isset($this->request->get['order'])) {
			$order = $this->request->get['order'];
		} else {
			$order = 'DESC';
		}

		if (isset($this->request->get['page'])) {
			$page = $this->request->get['page'];
		} else {
			$page = 1;
		}

		$url = '';

		if (isset($this->request->get['filter_order_id'])) {
			$url .= '&filter_order_id=' . $this->request->get['filter_order_id'];
		}

		if (isset($this->request->get['sort'])) {
			$url .= '&sort=' . $this->request->get['sort'];
		}

		if (isset($this->request->get['order'])) {
			$url .= '&order=' . $this->request->get['order'];
		}

		if (isset($this->request->get['page'])) {
			$url .= '&page=' . $this->request->get['page'];
		}

		$data['sort_order']  = $this->url->link('payment/mkb/transactions', 'token=' . $this->session->data['token'] . '&sort=o.OrderId' . $url, true);
		$data['sort_status'] = $this->url->link('payment/mkb/transactions', 'token=' . $this->session->data['token'] . '&sort=o.status' . $url, true);
		$data['sort_total']  = $this->url->link('payment/mkb/transactions', 'token=' . $this->session->data['token'] . '&sort=o.amt' . $url, true);

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => $this->language->get('text_home'),
			'href' => $this->url->link('common/dashboard', 'token=' . $this->session->data['token'], true),
		);

		$data['breadcrumbs'][] = array(
			'text' => $this->language->get('heading_title'),
			'href' => $this->url->link('payment/mkb/transactions', 'token=' . $this->session->data['token'] . $url, true),
		);

		$data['transactions'] = array();

		$filter_data = array(
			'filter_order_id' => $filter_order_id,
			'sort'            => $sort,
			'order'           => $order,
			'start'           => ($page - 1) * $this->config->get('config_limit_admin'),
			'limit'           => $this->config->get('config_limit_admin'),
		);

		$order_total = $this->model_payment_mkb_transactions->getTotalTransactions($filter_data);

		$results = $this->model_payment_mkb_transactions->getTransactions($filter_data);

		foreach ($results as $result) {
			$info = [
				'status'  => $this->language->get('transaction_status_' . $result['status']),
				'amt'     => round(intval(ltrim($result['amt'], '0')) / 100, 2),
				'refund'  => false,
				'capture' => false,
				'reverse' => false,
				'view'    => $this->url->link('sale/order/info', 'token=' . $this->session->data['token'] . '&order_id=' . $result['OrderID'] . $url, true),
				'edit'    => $this->url->link('sale/order/edit', 'token=' . $this->session->data['token'] . '&order_id=' . $result['OrderID'] . $url, true),
			];

			if ($result['status'] == Status::CAPTURE) {
				$info['refund'] = $this->url->link('payment/mkb/transactions/refund', 'token=' . $this->session->data['token'] . '&order_id=' . $result['OrderID'] . $url, true);
			}

			if ($result['status'] == Status::PREAUTH) {
				$info['capture'] = $this->url->link('payment/mkb/transactions/capture', 'token=' . $this->session->data['token'] . '&order_id=' . $result['OrderID'] . $url, true);
				$info['reverse'] = $this->url->link('payment/mkb/transactions/reverse', 'token=' . $this->session->data['token'] . '&order_id=' . $result['OrderID'] . $url, true);
			}

			$result                 = array_merge($result, $info);
			$data['transactions'][] = $result;
		}

		$data['heading_title'] = $this->language->get('heading_title');

		$data['text_list']           = $this->language->get('text_list');
		$data['text_no_results']     = $this->language->get('text_no_results');
		$data['text_confirm']        = $this->language->get('text_confirm');
		$data['text_missing']        = $this->language->get('text_missing');
		$data['text_loading']        = $this->language->get('text_loading');
		$data['text_card_saved']     = $this->language->get('text_card_saved');
		$data['text_refund_action']  = $this->language->get('text_refund_action');
		$data['text_capture_action'] = $this->language->get('text_capture_action');
		$data['text_reverse_action'] = $this->language->get('text_reverse_action');

		$data['column_order_id']       = $this->language->get('column_order_id');
		$data['column_status']         = $this->language->get('column_status');
		$data['column_total']          = $this->language->get('column_total');
		$data['column_action']         = $this->language->get('column_action');
		$data['column_card']           = $this->language->get('column_card');
		$data['column_payment_status'] = $this->language->get('column_payment_status');

		$data['entry_order_id'] = $this->language->get('entry_order_id');

		$data['button_invoice_print']  = $this->language->get('button_invoice_print');
		$data['button_shipping_print'] = $this->language->get('button_shipping_print');
		$data['button_add']            = $this->language->get('button_add');
		$data['button_edit']           = $this->language->get('button_edit');
		$data['button_delete']         = $this->language->get('button_delete');
		$data['button_filter']         = $this->language->get('button_filter');
		$data['button_view']           = $this->language->get('button_view');
		$data['button_ip_add']         = $this->language->get('button_ip_add');

		$data['token'] = $this->session->data['token'];

		$pagination        = new Pagination();
		$pagination->total = $order_total;
		$pagination->page  = $page;
		$pagination->limit = $this->config->get('config_limit_admin');
		$pagination->url   = $this->url->link('payment/mkb/transactions', 'token=' . $this->session->data['token'] . $url . '&page={page}', true);

		$data['pagination'] = $pagination->render();

		$data['results'] = sprintf($this->language->get('text_pagination'), ($order_total) ? (($page - 1) * $this->config->get('config_limit_admin')) + 1 : 0, ((($page - 1) * $this->config->get('config_limit_admin')) > ($order_total - $this->config->get('config_limit_admin'))) ? $order_total : ((($page - 1) * $this->config->get('config_limit_admin')) + $this->config->get('config_limit_admin')), $order_total, ceil($order_total / $this->config->get('config_limit_admin')));

		if (isset($this->error['warning'])) {
			$data['error_warning'] = $this->error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset($this->session->data['success'])) {
			$data['success'] = $this->session->data['success'];

			unset($this->session->data['success']);
		} else {
			$data['success'] = '';
		}

		if (isset($this->request->post['selected'])) {
			$data['selected'] = (array) $this->request->post['selected'];
		} else {
			$data['selected'] = array();
		}

		for ($i = 0; $i < 8; $i++) {
			$data['transaction_status_' . $i] = $this->language->get('transaction_status_' . $i);
		}

		$data['filter_order_id'] = $filter_order_id;
		$data['sort']            = $sort;
		$data['order']           = $order;

		$data['header']      = $this->load->controller('common/header');
		$data['column_left'] = $this->load->controller('common/column_left');
		$data['footer']      = $this->load->controller('common/footer');

		if (isset($this->session->data['success'])) {
			$data['success'] = $this->session->data['success'];

			unset($this->session->data['success']);
		} else {
			$data['success'] = '';
		}

		if (isset($this->session->data['error_warning'])) {
			$data['error_warning'] = $this->session->data['error_warning'];

			unset($this->session->data['error_warning']);
		} else {
			$data['error_warning'] = '';
		}

		$this->response->setOutput($this->load->view('payment/mkb/transactions.tpl', $data));
	}

	public function refund()
	{
		$this->load->language('payment/mkb/transactions');
		$orderId = $this->request->get['order_id'];

		$data = $this->prepareActionData($orderId);

		try {
			$data['Signature'] = Signature::calcFinActionSign($data, $this->config->get('mkb_secret'));
			$client            = new Client();
			if ($this->config->get('mkb_mode') == 'test') {
				$client->setTestMode();
			}
			$result       = $client->refundPayment($data);
			$responseCode = $result['ResponseCode'] ?? 0;
			if ($responseCode == 1) {
				$this->mkblib->updateTransactionStatus($orderId, Status::REFUND);
				$this->session->data['success'] = $this->language->get('text_action_success');
			}

		} catch (Exception $e) {
			$this->session->data['error_warning'] = $this->language->get('text_action_error') . $e->getMessage();
		}
		$this->response->redirect($this->url->link('payment/mkb/transactions', 'token=' . $this->session->data['token'], true));
	}

	public function reverse()
	{
		$this->load->language('payment/mkb/transactions');
		$orderId = $this->request->get['order_id'];

		$data = $this->prepareActionData($orderId);
		try {
			$data['Signature'] = Signature::calcFinActionSign($data, $this->config->get('mkb_secret'));
			$client            = new Client();
			if ($this->config->get('mkb_mode') == 'test') {
				$client->setTestMode();
			}
			$result       = $client->reversePayment($data);
			$responseCode = $result['ResponseCode'] ?? 0;
			if ($responseCode == 1) {
				$this->mkblib->updateTransactionStatus($orderId, Status::REVERSE);
				$this->session->data['success'] = $this->language->get('text_action_success');
			}

		} catch (Exception $e) {
			$this->session->data['error_warning'] = $this->language->get('text_action_error') . $e->getMessage();
		}
		$this->response->redirect($this->url->link('payment/mkb/transactions', 'token=' . $this->session->data['token'], true));
	}

	public function capture()
	{
		$this->load->language('payment/mkb/transactions');
		$orderId = $this->request->get['order_id'];

		$data = $this->prepareActionData($orderId);

		try {
			$data['Signature'] = Signature::calcFinActionSign($data, $this->config->get('mkb_secret'));
			$client            = new Client();
			if ($this->config->get('mkb_mode') == 'test') {
				$client->setTestMode();
			}
			$result       = $client->capturePayment($data);
			$responseCode = $result['ResponseCode'] ?? 0;
			if ($responseCode == 1) {
				$this->mkblib->updateTransactionStatus($orderId, Status::CAPTURE);
				$this->session->data['success'] = $this->language->get('text_action_success');
			}

		} catch (Exception $e) {
			$this->session->data['error_warning'] = $this->language->get('text_action_error') . $e->getMessage();
		}
		$this->response->redirect($this->url->link('payment/mkb/transactions', 'token=' . $this->session->data['token'], true));
	}

	private function prepareActionData($orderId)
	{
		$query   = $this->db->query("SELECT * FROM `" . DB_PREFIX . "mkb` WHERE OrderID=" . intval($orderId));
		$data    = $query->row;
		$query   = array(
			'MerID'               => $this->config->get('mkb_mid'),
			'AcqID'               => 443222,
			'PurchaseAmt'         => $data['amt'],
			'OrderID'             => $data['OrderID'],
			'AuthorizationNumber' => substr($data['AuthCode'], 0, 6),
			'Amount'              => $data['amt'],
		);
		return $query;
	}

}
