<?php

use Mkb\Sdk\Signature;
use Mkb\Sdk\Helper;

class ControllerPaymentmkb extends Controller
{
	private $mkblib = null;

	public function __construct($registry) {

		$this->registry = $registry;
		if (method_exists($this->load, "library")) {
			$this->load->library('mkblib');
		}
		$this->mkblib = new Mkblib($this->registry);
	}

	private function oldversion(){
		return version_compare(VERSION, "2.2.0.0") == -1;
	}

	public function index()
	{

		$this->load->model('checkout/order');
		$this->load->model('payment/mkb');

		$data = [
			'gateway_url'    => Helper::getMpiUrl($this->config->get('mkb_mode') == 'test'),
			'button_confirm' => $this->language->get('button_confirm'),
			'text_loading'   => $this->language->get('text_loading'),
		];

		if($this->oldversion()) {
			if (file_exists(DIR_TEMPLATE . $this->config->get('config_template') . '/template/payment/mkb.tpl')) {
				return $this->load->view($this->config->get('config_template') . '/template/payment/mkb.tpl', $data);
			} else {
				return $this->load->view('default/template/payment/mkb.tpl', $data);
			}
		} else {
			return $this->load->view('payment/mkb', $data);
		}
	}

	public function send()
	{
		$this->load->model('checkout/order');
		$this->load->model('payment/mkb');

		$orderId   = (int) $this->session->data['order_id'];
		$orderInfo = $this->model_checkout_order->getOrder($orderId);

		$amount = $this->currency->format($orderInfo['total'], $orderInfo['currency_code'], $orderInfo['currency_value'], false);
		$amount = round($amount * 100);
		$amount = sprintf('%012d', $amount);

		$data = [
			'mid'           => $this->config->get('mkb_mid'),
			'aid'           => 443222,
			'amount'        => $amount,
			'oid'           => $orderId,
			'redirect_url'  => $this->url->link('checkout/success', '', true),
			'cancel_link'   => $this->url->link('checkout/checkout', '', true),
			'directposturl' => $this->url->link('payment/mkb/callback', '', true),
			'client_mail'   => $orderInfo['email'],
		];

		if ($this->config->get('mkb_merchant_mail')) {
			$data['merchant_mail'] = $this->config->get('mkb_merchant_mail');
		}

		$orderStatusId = $this->config->get('config_order_status_id');
		$this->model_checkout_order->addOrderHistory($orderId, $orderStatusId, '');

		$data['signature'] = Signature::calcPaymentFormSign($data, $this->config->get('mkb_secret'));

		echo json_encode([
			'error'  => false,
			'fields' => $data,
		]);
		exit();
	}

	public function callback()
	{
		$this->load->model('checkout/order');
		$post = $_POST;
		$signData = [
			$this->config->get('mkb_secret'),
			$post['MerID'],
			$post['AcqID'],
			$post['OrderID'],
			$post['ResponseCode'],
			$post['ReasonCode'],
		];

		$signature = Signature::calcPaymentNotifySign($post, $this->config->get('mkb_secret'));
		if (strcmp($post['Signature'], $signature) !== 0) {
			$this->error('Wrong Signature');
		}

		$info           = $this->mkblib->getTransactionInfo($post['OrderID']);
		$post['status'] = substr($info['status'], 0, 1);
		$this->mkblib->saveTransaction($post);

		$orderId = $post['OrderID'];

		if (strcmp($post['MerID'], $this->config->get('mkb_mid')) !== 0) {
			$this->error('Wrong merchant');
		}

		$orderInfo = $this->model_checkout_order->getOrder($orderId);

		if (!$orderInfo) {
			$this->error('Incorrect order id');
		}

		if ($post['ResponseCode'] !== '1') {
			return false;
		}

		if ($orderInfo['order_status_id'] == $this->config->get('mkb_confirm_order_status_id')) {
			return false;
		}

		$this->load->model('checkout/order');

		$this->model_checkout_order->addOrderHistory($orderId, $this->config->get('mkb_order_status_id'));
	}


	private function error($msg)
	{
		throw new Exception($msg, 500);
	}
}
