<?php
namespace Mkb\Sdk;

use Mkb\Sdk\Exceptions\EmptyPropertyException;
use Mkb\Sdk\Exceptions\InvalidPropertyValueException;

class Signature {

	public static function calc($data)
	{
		return base64_encode(hex2bin(hash('sha256', implode('', $data))));
	}

	public static function calcPaymentFormSign($data, $key)
	{
		if (empty($data['mid'])) {
			throw new EmptyPropertyException('Empty mid', 0, 'mid');
		}
		if (empty($data['aid'])) {
			throw new EmptyPropertyException('Empty aid', 0, 'aid');
		}
		if (empty($data['oid'])) {
			throw new EmptyPropertyException('Empty oid', 0, 'oid');
		}
		if (empty($data['amount'])) {
			throw new EmptyPropertyException('Empty amount', 0, 'amount');
		}
		if (!preg_match('#\d{12}#', $data['amount'])) {
			throw new InvalidPropertyValueException("Amount must be 12 digits", 0, 'amount', $data['amount']);
		}

		$signData = [
			$key,
			$data['mid'],
			$data['aid'],
			$data['oid'],
			$data['amount'],
			'643',
		];
		return self::calc($signData);
	}

	public static function calcPaymentNotifySign($data, $key)
	{
		if (empty($data['MerID'])) {
			throw new EmptyPropertyException('Empty MerID', 0, 'MerID');
		}
		if (empty($data['AcqID'])) {
			throw new EmptyPropertyException('Empty AcqID', 0, 'AcqID');
		}
		if (empty($data['OrderID'])) {
			throw new EmptyPropertyException('Empty OrderID', 0, 'OrderID');
		}
		if (empty($data['ResponseCode'])) {
			throw new EmptyPropertyException('Empty ResponseCode', 0, 'ResponseCode');
		}
		if (empty($data['ReasonCode'])) {
			throw new EmptyPropertyException('Empty ReasonCode', 0, 'ReasonCode');
		}

		$signData = [
			$key,
			$data['MerID'],
			$data['AcqID'],
			$data['OrderID'],
			$data['ResponseCode'],
			$data['ReasonCode'],
		];
		return self::calc($signData);
	}

	public static function calcReceiptRequestSign($data, $key)
	{
		$signData = [
			$key,
			$data['merchant'],
			$data['id'],
		];
		return self::calc($signData);
	}

	public static function calcActionSign($mid, $aid, $key)
	{
		$signData = [
			$key,
			$mid,
			$aid,
		];
		return self::calc($signData);
	}

	public static function calcFinActionSign($data, $key)
	{
		if (empty($data['MerID'])) {
			throw new EmptyPropertyException('Empty MerID', 0, 'MerID');
		}
		if (empty($data['AcqID'])) {
			throw new EmptyPropertyException('Empty AcqID', 0, 'AcqID');
		}
		if (empty($data['OrderID'])) {
			throw new EmptyPropertyException('Empty OrderID', 0, 'OrderID');
		}
		if (empty($data['PurchaseAmt'])) {
			throw new EmptyPropertyException('Empty amount', 0, 'PurchaseAmt');
		}
		if (!preg_match('#\d{12}#', $data['PurchaseAmt'])) {
			throw new InvalidPropertyValueException("PurchaseAmt must be 12 digits", 0, 'PurchaseAmt', $data['PurchaseAmt']);
		}

		if (!preg_match('#\d{12}#', $data['Amount'])) {
			throw new InvalidPropertyValueException("Amount must be 12 digits", 0, 'Amount', $data['Amount']);
		}

		$signData = [
			$key,
			$data['MerID'],
			$data['AcqID'],
			$data['OrderID'],
			$data['PurchaseAmt'],
			'643',
		];
		return self::calc($signData);
	}
}